# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=C0103,W0703,W0201,W0612,W0142,E0201
# pylint-version = 0.7.0
#
# Copyright 2004-2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.differ

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

import unittest
_tests = []

def findTestData():
    """ finds the testdata directory """
    import os
    data_path = os.environ.get('TESTDATA')
    if data_path:
        data_path = os.path.abspath(data_path)
        if os.path.isdir(data_path):
            return data_path

    return None


class importTestCase(unittest.TestCase):
    """ Importing svnmailer.differ """

    def testImport(self):
        """ from svnmailer import differ """
        try:
            from svnmailer import differ
        except Exception, exc:
            self.fail("svnmailer.differ could not be imported: %s" % str(exc))

_tests.append(importTestCase)


class DifferTestBase(unittest.TestCase):
    """ Base testign class for the differ """

    def doStringDiff(self):
        """ string differ """
        expected = [
            "--- label1 date1",
            "+++ label2 date2",
            "@@ -1,2 +1,2 @@",
            "-alpha",
            "-beta",
            "+gamma",
            "+delta",
        ]
        args = [
            "alpha\nbeta\n", "gamma\ndelta\n", "label1", "label2",
            "date1", "date2"
        ]
        result = [line.rstrip() for line in self._differ.getStringDiff(*args)]

        self.failUnlessEqual(
            expected, result,
            "getStringDiff(%s) yields %r" % (repr(args)[1:-1], result)
        )

    def doFileDiff(self):
        """ file differ """
        import os

        datadir = findTestData()
        expected = [line.rstrip() for line in
            file(os.path.join(datadir, 'diff12.txt'), 'rb').readlines()
        ]
        args = [
            os.path.join(datadir, "file1.txt"),
            os.path.join(datadir, "file2.txt"),
            "label1", "label2", "date1", "date2"
        ]
        result = [line.rstrip() for line in self._differ.getFileDiff(*args)]

        self.failUnlessEqual(
            expected, result,
            "getFileDiff(%s) yields %r" % (repr(args)[1:-1], result)
        )


class InternalDifferTestCase(DifferTestBase):
    """ svnmailer.differ.InternalDiffer """

    def setUp(self):
        """ get the differ """
        from svnmailer import differ
        self._differ = differ.InternalDiffer()

    def test00StringDiff(self):
        """ Internal string differ works """
        DifferTestBase.doStringDiff(self)

    if not findTestData():
        def test01FileDiff(self):
            """ (no testdata found) Internal File differ SKIPPED """
            pass
    else:
        def test01FileDiff(self):
            """ Internal file differ works """
            DifferTestBase.doFileDiff(self)

_tests.append(InternalDifferTestCase)


class ExternalDifferTestCase(DifferTestBase):
    """ svnmailer.differ.InternalDiffer """

    def test00getPipe(self):
        """ External differ call works and passes labels """
        import sys
        from svnmailer import differ as _differ

        cmd = [sys.executable, '-c',
            'import sys; sys.stdout.write(repr(sys.argv[1:]))',
            '%(label_from)s', '%(label_to)s', '%(from)s', '%(to)s'
        ]
        differ = _differ.ExternalDiffer(cmd)
        expected = repr(['label1 date1', 'label2 date2', 'file1', 'file2'])
        args = ['file1', 'file2', 'label1', 'label2', 'date1', 'date2']
        pipe = differ._getPipe(*args)
        try:
            result = pipe.fromchild.read()
        finally:
            pipe.wait()

        self.failUnlessEqual(
            expected, result,
            "external program sent %r" % result
        )

    def _getDiffCommand(self):
        """ We build our own external differ """
        import sys
        return [sys.executable, '-c',
            r'import difflib, sys; c1=file(sys.argv[3],"rb").readlines();'
            r'c2=file(sys.argv[4],"rb").readlines(); sys.stdout.write('
            r'"".join([line for line in difflib.unified_diff('
            r'c1, c2, sys.argv[1], sys.argv[2])]))',
            '%(label_from)s', '%(label_to)s', '%(from)s', '%(to)s'
        ]

    def test01StringDiff(self):
        """ External string differ works """
        from svnmailer import differ
        self._differ = differ.ExternalDiffer(self._getDiffCommand())
        DifferTestBase.doStringDiff(self)

    if not findTestData():
        def test02FileDiff(self):
            """ (no testdata found) External file differ SKIPPED """
    else:
        def test02FileDiff(self):
            """ External file differ works """
            from svnmailer import differ
            self._differ = differ.ExternalDiffer(self._getDiffCommand())
            DifferTestBase.doFileDiff(self)

_tests.append(ExternalDifferTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
