# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=C0103,W0703,W0612,R0904
# pylint-version = 0.7.0
#
# Copyright 2004-2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Testcases for svnmailer.stream

:var _tests: The test classes
:type _tests: ``list``
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = ['getSuite']

import unittest
_tests = []


class importTestCase(unittest.TestCase):
    """ Importing svnmailer.stream """

    def testImport(self):
        """ from svnmailer import stream """
        try:
            from svnmailer import stream
        except Exception, exc:
            self.fail("svnmailer.stream could not be imported: %s" % str(exc))

_tests.append(importTestCase)


class streamTestBase(unittest.TestCase):
    """ Base class for stream tests """

    def setUp(self):
        """ import the stream module """
        from svnmailer import stream
        self._stream = stream


class UnicodeStreamTestCase(streamTestBase):
    """ svnmailer.stream.UnicodeStream """

    def test00FileObject(self):
        """ UnicodeStream returns a file like object """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.UnicodeStream(orig)
        writer.write("foo")
        writer.close()

    def test01Accept(self):
        """ UnicodeStream accepts unicode and strings """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.UnicodeStream(orig)
        writer.write(u"\u20acuro\n")
        writer.write(u"\u20acuro\n".encode('utf-8'))
        result = orig.getvalue()
        writer.close()
        expected = "\xe2\x82\xacuro\n\xe2\x82\xacuro\n"
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

    def test02Replace(self):
        """ UnicodeStream replaces miscoded characters """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.UnicodeStream(orig, "us-ascii", "iso-8859-1")
        writer.write(u"\u20acuro\n")
        writer.write(u"\u20acuro\n".encode('utf-8'))
        result = orig.getvalue()
        writer.close()
        expected = "?uro\n???uro\n"
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

_tests.append(UnicodeStreamTestCase)


class TruncatingStreamTestCase(streamTestBase):
    """ svnmailer.stream.TruncatingStream """

    def test00FileObject(self):
        """ TruncatingStream returns a file like object """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.TruncatingStream(orig, 20)
        writer.write("foo")
        writer.close()

    def test01Accept(self):
        """ TruncatingStream accepts strings up to the limit """
        import cStringIO

        for num in range(1, 21):
            expected = "\n" * num
            writer = self._stream.TruncatingStream(cStringIO.StringIO(), 20)
            writer.write(expected)
            result = writer.getvalue()
            count = writer.getTruncatedLineCount()
            writer.close()

            self.failUnlessEqual(expected, result,
                "Didn't get %r, but %r" % (expected, result)
            )
            expected = 0
            result = count
            self.failUnlessEqual(expected, result,
                "Didn't get %r, but %r" % (expected, result)
            )

    def test02Truncate(self):
        """ TruncatingStream chops after reaching the limit on line basis """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.TruncatingStream(orig, 20)
        writer.write('1234567890\n1234567890\n')
        expected = '1234567890\n'
        result = writer.getvalue()
        writer.close()
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

    def test03LineCount(self):
        """ TruncatingStream returns the correct truncated line count """
        import cStringIO

        tests = (
            (2, '1234567890\n1234567890\n123456789'),
            (2, '1234567890\n1234567890\n123456789\n'),
            (3, '1234567890\n1234567890\n123456789\n1'),
            (1, '1234567890\n1234567890\n'),
            (1, '1234567890\n1234567890'),
        )
        for expected, data in tests:
            orig = cStringIO.StringIO()
            writer = self._stream.TruncatingStream(orig, 20)
            writer.write(data)
            result = writer.getTruncatedLineCount()
            writer.close()
            self.failUnlessEqual(expected, result,
                "Didn't get %r, but %r" % (expected, result)
            )

    def test04NoTruncate(self):
        """ TruncatingStream allows writing without truncation """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.TruncatingStream(orig, 20)
        writer.write('1234567890\n1234567890\n')
        writer.writeWithoutTruncation('foo')
        result = writer.getvalue()
        expected = '1234567890\nfoo'
        writer.close()
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

    def test05seek(self):
        """ TruncatingStream allows seeking to position 0 """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.TruncatingStream(orig, 20)
        writer.write('1234567890\n1234567890\n')
        writer.seek(0)
        writer.truncate()
        writer.write('foo')
        expected = 'foo'
        result = writer.getvalue()
        writer.close()
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

    def test06getvalue(self):
        """ TruncatingStream adds a note, if requested """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.TruncatingStream(orig, 20, True)
        writer.write('1234567890\n1234567890\n')
        expected = '1234567890\n\n[... 1 lines stripped ...]\n'
        result = writer.getvalue()
        writer.close()
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

_tests.append(TruncatingStreamTestCase)


class CuckooStreamTestCase(streamTestBase):
    """ svnmailer.stream.CuckooStream """

    def test00FileObject(self):
        """ CuckooStream returns a file like object """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.CuckooStream(orig)
        writer.write("foo")
        writer.close()

    def test01Replace(self):
        """ CuckooStream replaces the stream on demand """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.CuckooStream(orig)
        writer.write("foo")
        fake = cStringIO.StringIO()
        fake.write("bar")
        writer.replaceStream(fake)
        result = writer.getvalue()
        writer.close()
        expected = "bar"
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

_tests.append(CuckooStreamTestCase)


class SplittingStreamTestCase(streamTestBase):
    """ svnmailer.stream.SplittingStream """

    def test00FileObject(self):
        """ SplittingStream returns a file like object """
        import cStringIO

        orig = cStringIO.StringIO()
        writer = self._stream.SplittingStream(orig)
        writer.write("foo")
        writer.close()

    def test01split(self):
        """ SplittingStream splits the stream on demand """
        writer = self._stream.SplittingStream()
        writer.write("foo")
        writer.split()
        writer.write("bar")
        writer.split()
        writer.split()
        result = writer.getvalue()
        count = writer.getPartCount()
        writer.close()
        expected = ""
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )
        expected = 2
        result = count
        self.failUnlessEqual(expected, result,
            "Didn't get %r, but %r" % (expected, result)
        )

    def test02parts(self):
        """ SplittingStream splits the stream on demand """
        writer = self._stream.SplittingStream()
        writer.write("foo")
        writer.split()
        writer.write("bar")
        writer.split()
        writer.split()
        for idx, expected in enumerate(['foo', 'bar']):
            result = writer.getPart(idx)
            self.failUnlessEqual(expected, result,
                "Didn't get %r, but %r" % (expected, result)
            )

_tests.append(SplittingStreamTestCase)


#### Glue code                                      ####
########################################################
__all__.extend(_tests)

def getSuite():
    """ Returns the test suite for this module """
    return [_test.__name__ for _test in _tests]
