# -*- coding: iso-8859-1 -*-
# pylint: disable-msg=C0103,W0613
# pylint-version = 0.7.0
#
# Copyright 2005 Andr Malo or his licensors, as applicable
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Test utilities
"""
__author__    = "Andr Malo"
__docformat__ = "restructuredtext en"
__all__       = [
    'findTestData', 'mockModule', 'unmockModule', 'TestBase'
]

# global imports -> now the modules are loaded and can be mocked
import imp, os, sys
from svnmailer_test import _unittest


def findTestData():
    """ finds the testdata directory """
    data_path = os.environ.get('TESTDATA')
    if data_path:
        data_path = os.path.abspath(data_path)
        if os.path.isdir(data_path):
            return data_path

    return None


class Importer(object):
    """ Importer, see `PEP 302`_

        :IVariables:
         - `name`: The full dotted name of the module, but only if
           the mock is not sticky (``None`` otherwise)
         - `_name`: The full dotted name of the module
         - `_mdict`: The initial dict
         - `_ispkg`: Is it a package?
         - `_pkgpath`: The package path
         - `_sticky`: Permanent mock?

        :Types:
         - `name`: ``str``
         - `_name`: ``str``
         - `_mdict`: ``dict``
         - `_ispkg`: ``bool``
         - `_pkgpath`: ``str``
         - `_sticky`: ``bool``

        .. _PEP 302: http://www.python.org/peps/pep-0302.html
    """

    def __init__(self, fullname, mdict, ispkg, pkgpath, sticky):
        """ Initialization

            :Parameters:
             - `fullname`: The full dotted name of the module
             - `mdict`: The initial dict
             - `ispkg`: Is it a package?
             - `pkgpath`: The package path (``['path', ...]``)
             - `sticky`: Permanent mock? If ``True``, `unmockModule` cannot
               unmock it. Useful for virtualized packages.

            :Types:
             - `fullname`: ``str``
             - `mdict`: ``dict``
             - `ispkg`: ``bool``
             - `pkgpath`: ``list``
             - `sticky`: ``bool``
        """
        self._name = fullname
        if sticky:
            self.name = None
        else:
            self.name = fullname
        self._mdict = mdict
        self._ispkg = ispkg
        self._pkgpath = pkgpath
        self._sticky = sticky


    def find_module(self, fullname, path = None):
        """ Returns the loader instance

            This is an interface method of `PEP 302`_

            :Parameters:
             - `fullname`: The dotted.name of the module
             - `path`: The package path (unused)

            :Types:
             - `fullname`: ``str``
             - `path`: ``str``

            :return: The loader instance (`self`), if this
                     `Importer` matches `fullname`; or ``None``
            :rtype: `Importer`

            .. _PEP 302: http://www.python.org/peps/pep-0302.html
        """
        if fullname == self._name:
            return self

        return None


    def load_module(self, fullname):
        """ Returns the module instance

            This is an interface method of `PEP 302`_

            :param fullname: The dotted.name of the module
            :type fullname: ``str``

            :return: The module object
            :rtype: ``module``

            :Exceptions:
             - `RuntimeError`: The module already exists
             - `ImportError`: Forced, if the initial dict is ``None``

            .. _PEP 302: http://www.python.org/peps/pep-0302.html
        """
        if sys.modules.has_key(fullname):
            raise RuntimeError("module %r already exists?!" % fullname)

        if self._mdict is None:
            raise ImportError("Mocked ImportError for %s" % fullname)

        mod = sys.modules[fullname] = imp.new_module(fullname)
        mod.__file__ = '<mocked %s %s' % (
            ["module", "package"][self._ispkg], fullname
        )
        mod.__loader__ = self
        if self._ispkg:
            mod.__path__ = self._pkgpath or []
        if self._mdict:
            mod.__dict__.update(self._mdict)

        return mod


def mockModule(name, mdict = None, ispkg = False, pkgpath = None,
               sticky = False):
    """ Mocks a module

        The mocking is done by inserting a proper `Importer` instance into
        `sys.meta_path`.

        :Parameters:
         - `name`: The full dotted name of the module
         - `mdict`: The initial dict
         - `ispkg`: Is it a package?
         - `pkgpath`: The package path (``['path', ...]``)
         - `sticky`: Permanent mock? If ``True``, `unmockModule` cannot
           unmock it. Useful for virtualized packages.

        :Types:
         - `name`: ``str``
         - `mdict`: ``dict``
         - `ispkg`: ``bool``
         - `pkgpath`: ``list``
         - `sticky`: ``bool``
    """
    sys.meta_path.insert(0, Importer(name, mdict, ispkg, pkgpath, sticky))


def unmockModule(name):
    """ Unmocks a module

        Actually it removes any global instance of the module `name`,
        may it be mocked or not. Any `Importer` objects hanging around
        in `sys.meta_path` for `name` are also removed.

        :param name: The dotted.name of the module
        :type name: ``str``
    """
    # remove mock loader
    importers = [importer for importer in sys.meta_path
        if hasattr(importer, 'name') and importer.name == name
    ]
    for importer in importers:
        sys.meta_path.remove(importer)

    # remove from global sys.modules dict
    try:
        del sys.modules[name]
    except KeyError:
        """ ignore """
        pass

    # drop it also from the parent module
    idx = name.rfind('.')
    if idx != -1:
        parent = name[:idx]
        submodule = name[idx + 1:]
        try:
            del sys.modules[parent].__dict__[submodule]
        except KeyError:
            """ ok """
            pass


class MockStore(object):
    """ Simple data container

        :IVariables:
         - `result`: List, where expectations can be stored
         - `error`: Error condition, which can be evaluated by
           mocked objects (typically just a string)
         - `_mocker`: `Mocker` instance

        :Types:
         - `result`: ``list``
         - `error`: anything
         - `_mocker`: `Mocker`
    """

    def __init__(self, **kwargs):
        """ Initialization """
        self.result = []
        self.error = None
        self._mocker = Mocker()


    def __getitem__(self, name):
        """ Returns a specific mocked module object

            :param name: The dotted.name of the module
            :type name: ``str``

            :return: The module object
            :rtype: ``module``

            :exception KeyError: The module was not found
        """
        return self._mocker[name]


class Mocker(object):
    """ Mock loader and unloader class

        :IVariables:
         - `_mods`: dictionary holding the mocked module names and object
           (``{'dotted.name': moduleobj, ...}``)
         - `_mocked`: List holding the names of the mocked modules in the
           order they were mocked.

        :Types:
         - `_mods`: ``dict``
         - `_mocked`: ``list``
    """

    def __init__(self):
        """ Initialization """
        self._mods = {}
        self._mocked = []


    def __getitem__(self, name):
        """ Returns a specific mocked module object

            :param name: The dotted.name of the module
            :type name: ``str``

            :return: The module object
            :rtype: ``module``

            :exception KeyError: The module was not found
        """
        return self._mods[name]


    def addMock(self, name, initial, ispkg = False):
        """ Mocks one module or package

            :Parameters:
             - `name`: The dotted.name of the module
             - `initial`: The `initial` parameter contains the initial
               dictionary of the module. If it is ``None``, importing
               the module in the tested code causes an ``ImportError``
             - `ispkg`: Is it a package?

            :Types:
             - `name`: ``str``
             - `initial`: ``dict``
             - `ispkg`: ``bool``

            :return: The mocked module object
            :rtype: ``module``
        """
        self._mocked.append(name)
        self._mods[name] = mod = self._mockModule(name, initial, ispkg)

        return mod


    def destroy(self):
        """ Unmocks all mocked modules """
        self._mods.clear()
        while self._mocked:
            self._unmockModule(self._mocked.pop())


    def mockStarPath(self, name):
        """ Mocks the path to the module `name`

            If `name` is a dotted name (like ``distutils.core``),
            by default only the part after the last dot is mocked (``core`` in
            this case). However, it can be useful to emulate package path
            partially or at all. You do this by putting an asterisk (``*``) in
            front of the first part which should be mocked. For example, a
            name of ``*distutils.command.install`` would mock the following:

            1. the distutils *package*
            2. the distutils.command *package*
            3. the distutils.command.install *module*

            Furthermore, a name of ``distutils.*command.install`` would leave
            out step 1 and fail if the ``distutils`` package was not available.

            Note that this method only resolves the packages below the actual
            module, i.e. steps 1 and 2 above.

            :param name: The name of the final module
            :type name: ``str``

            :return: The clean name of the module (without any asterisk)
            :rtype: ``str``
        """
        parts = name.split('.')
        cleanparts = [
            [part, part[1:]][part.startswith('*')]
            for part in parts
        ]
        cleanname = '.'.join(cleanparts)

        if not self._mods.has_key(cleanname):
            mocking = False
            for idx, part in enumerate(parts[:-1]):
                if not(mocking or part.startswith('*')):
                    continue
                mocking = True
                modnow = '.'.join(cleanparts[:idx + 1])
                if not self._mods.has_key(modnow):
                    self.addMock(modnow, {}, True)

        return cleanname


    def _mockModule(self, name, mdict, ispkg = False):
        """ Mocks module `name` using module dict `mdict`

            The method takes care of unloading a possibly existing
            global instance of the module in question first.

            :Parameters:
             - `name`: The (possibly dotted) name of the module
               (only the last part ist mocked)
             - `mdict`: initial dictionary of the module
             - `ispkg`: Is it a package?

            :Types:
             - `name`: ``str``
             - `mdict`: ``dict``
             - `ispkg`: ``bool``

            :Exceptions:
             - `ImportError`: Some bad error occured while mocking the
               module (should not happen...)
             - `RuntimeError`: The global module instance could not be
               unloaded or the module could not be mocked for some reason
        """
        unmockModule(name)
        mockModule(name, mdict, ispkg)

        try:
            mod = _unittest.loadModule(name)
        except ImportError:
            if mdict is not None:
                raise
            mod = None
        else:
            if 'mocked' not in mod.__file__:
                raise RuntimeError("'%s' is %r?!" % (name, mod))

        return mod


    def _unmockModule(self, name):
        """ Unmocks module <name>

            :param name: The dotted.name of the module
            :type name: ``str``

            :exception RuntimeError: The module could not be unmocked for some
                                     reason
        """
        unmockModule(name)

        try:
            mfile = _unittest.loadModule(name).__file__
        except ImportError:
            """ The real module prolly doesn't exist """
            pass
        except AttributeError:
            """ no ``__file__`` attribute is a good sign as well """
            pass
        else:
            if 'mocked' in mfile:
                raise RuntimeError("module '%s' was not unmocked" % name)

        unmockModule(name)


class TestBase(_unittest.TestCase):
    """ Base class for tests, which provides some convenience

        :CVariables:
         - `PREFIX`: The prefix that should prepend the test's doc string
           if the docstring begins with a dot. ``None`` if there's no prefix.

         - `ENCODING`: The character encoding of the test's docstrings.
           ``None`` if they shouldn't be recoded for terminal output

         - `LOAD_MODULES`: A list of modules, which should be preloaded before
           the test starts. The module objects are added to the test context.
           Each list entry is a ``tuple`` of two items. The first item contains
           the name of the attribute, which should store the module object,
           whose dotted.name is stored in the second item. `LOAD_MODULES` is
           typically used to load the tested module(s).
           (``(('attr', 'dottedname'), ...)``)

        :IVariables:
         - `mock`: `MockStore` instance, where all mocks can store parameters.

        :Types:
         - `PREFIX`: ``str``
         - `ENCODING`: ``str``
         - `LOAD_MODULES`: ``tuple``
         - `mock`: `MockStore`
    """
    PREFIX = None
    ENCODING = None
    LOAD_MODULES = None

    def shortDescription(self):
        """ Prefixes the docstring and applies `ENCODING` """
        desc = super(TestBase, self).shortDescription()
        if self.PREFIX and desc and desc.startswith('.'):
            desc = "%s%s" % (self.PREFIX, desc)

        if self.ENCODING is not None:
            desc = unicode(desc, self.ENCODING)

        return desc


    def setUp(self):
        """ Setup mock environment and load modules in `LOAD_MODULES` """
        self.mock = MockStore()
        for tup in self.getInitialMocks():
            self.addMock(*tup)

        if self.LOAD_MODULES:
            for attr, modname in self.LOAD_MODULES:
                mod = _unittest.loadModule(modname)
                reload(mod) # make sure to initialize fresh
                setattr(self, attr, mod)


    def tearDown(self):
        """ Unload mocked modules and refresh modules in `LOAD_MODULES` """
        if self.LOAD_MODULES:
            for attr, modname in self.LOAD_MODULES:
                delattr(self, attr)

        self.mock._mocker.destroy()

        # leave clean modules around
        if self.LOAD_MODULES:
            for attr, modname in self.LOAD_MODULES:
                try:
                    reload(_unittest.loadModule(modname))
                except (ImportError, AttributeError):
                    """ this may happen, but it's prolly ok """
                    pass


    def expect(self, *expected):
        """ Shortcut for ``self.failUnlessEqual(expected, self.mock.result)``

            :param expected: Expected entries in ``self.mock.result``
            :type expected: ``tuple``

            :exception failureException: Expectation failed
        """
        result = tuple(self.mock.result)
        self.failUnlessEqual(expected, result,
            "Expected %r, got %r" % (expected, result)
        )


    def public(self, module):
        """ Returns the public symbols of a module

            :param module: The module object
            :type module: ``module``

            :return: The list of public symbols
            :rtype: ``tuple``
        """
        if hasattr(module, '__all__'):
            return tuple(module.__all__)

        return tuple([
            name for name in vars(module) if not name.startswith('_')
        ])


    def getInitialMocks(self):
        """ Returns the list of initially mocked modules

            Each entry in the list consists of a tuple with two or three
            items, which will be passed to the `addMock` method. See there
            for details. The modules are mocked in order of appearance in
            the list and unmocked reversed.

            You can overide this method to declare initially mocked modules,
            the default list is empty.

            :return: List of initially mocked modules
                     (``[('name', dict(initial), ispkg = False), ...]``)
            :rtype: ``list``
        """
        return []


    def addMock(self, name, initial, ispkg = False):
        """ Mocks one module or package

            The module `name` is mocked and added to the internal list
            of mocked modules. The `tearDown` method takes care of
            unmocking all modules on this list after the test. If `name`
            is a dotted name (like ``distutils.core``), by default only
            the part after the last dot is mocked (``core`` in this case).
            However, it can be useful to emulate package path partially or
            at all. You do this by putting an asterisk (``*``) in
            front of the first part which should be mocked. For example, a
            name of ``*distutils.command.install`` would mock the following:

            1. the distutils package
            2. the distutils.command package
            3. the distutils.command.install module with `initial` as
               initial dictionary

            Furthermore, a name of ``distutils.*command.install`` would leave
            out step 1 and fail if the distutils package was not available.

            The `initial` parameter contains the initial dictionary of the
            module. If it is ``None``, importing the module in the tested
            code causes an ``ImportError``. This is useful for testing
            stuff like this::

                try:
                    import foo
                except ImportError:
                    import bar as foo

            If the mocked name should be semantically a package (i.e. may
            have submodules), `ispkg` needs to be set ``True``.

            :Parameters:
             - `name`: The dotted.name of the module
             - `initial`: Initial dict of the module
             - `ispkg`: Is it a package?

            :Types:
             - `name`: ``str``
             - `initial`: ``dict``
             - `ispkg`: ``bool``
        """
        if '*' in name:
            name = self.mock._mocker.mockStarPath(name)

        return self.mock._mocker.addMock(name, initial, ispkg)
